var collapsed = {};
var lastCollapsed = {};
var lockinfoDIV = document.getElementById("lockinfo");
var clockDIV;
var monthDIV
var weatherDIV;
var mailDIV;
var calendarDIV;
var callsDIV;
var voicemailDIV;
var smsDIV;
var clockTimer;
var dayTimer;

function relativeTime(sectionDIV, id, time, daysOnly, upperFirst, emptyString){
	var div = document.getElementById(id);
	if(!div || !sectionDIV){
		return false;
	}
	if(!time){
		var time = new Date();
	}
	addClass(div, "relativeTime");

	window.clearTimeout(div.timer);
	if(displayRelativeTimes){
		var string = time.relative(daysOnly);
		if(string){
			div.innerHTML = upperFirst ? string.upperFirst() : string;
		}else if(emptyString){
			div.innerHTML = emptyString;
		}
		div.timer = window.setTimeout(function(){relativeTime(sectionDIV, id, time, daysOnly, upperFirst);}, time.relative(daysOnly, true));
		if(!sectionDIV.relativeTimers){
			sectionDIV.relativeTimers = [div.timer];
		}else{
			sectionDIV.relativeTimers.push(div.timer);
		}
	}else{
		div.innerHTML = time.format(time.isSameDay() ? format_time : format_date_time_short);
	}
}

function clearRelativeTimers(sectionDIV){
	if(sectionDIV && sectionDIV.relativeTimers){
		for(var i = 0; i < sectionDIV.relativeTimers.length; i++){
			window.clearTimeout(sectionDIV.relativeTimers[i]);
		}
		sectionDIV.relativeTimers = [];
	}
}

function getSectionHeight(div){
	var h = 0;
	for(var i in div.lastChild.childNodes){
		h += div.lastChild.childNodes[i].clientHeight || 0;
	}
	return h;
}

function catchSwipe(event, tapFunction, args){
	var args = args || null;
	if(event && event.touches && event.touches.length == 1){
		event.stopPropagation();
		var startX = event.touches[0].pageX;
		var startY = event.touches[0].pageY;
		var x = startX;
		var y = startY;

		document.ontouchend = function(){
			if(typeof(tapFunction) == "function" && Math.sqrt(Math.pow(startX - x, 2) + Math.pow(startY -y, 2)) <= tap_maxDistance){
				tapFunction(args);
			}
			document.ontouchend = undefined;
			document.ontouchmove = undefined;
		};

		document.ontouchmove = function(e){
			if(Math.abs(startY - e.touches[0].pageY) > swipe_maxY){
				x = e.touches[0].pageX;
				y = e.touches[0].pageY;
				document.ontouchend(e);
				return false;
			}else if(Math.abs(startX - x) >= swipe_minDistance){
				if(Math.abs(y - e.touches[0].pageY) <= swipe_maxY){
					if(globalHide && startX < x && globalHide > 0 && !isGlobalCollapsed() || startX > x && globalHide < 0){
						toggleHide();
					}else if(globalCollapse){
						toggleAllSections();
					}
				}
				x = e.touches[0].pageX;
				y = e.touches[0].pageY;
				document.ontouchend(e);
			}else{
				x = e.touches[0].pageX;
				y = e.touches[0].pageY;
			}
		};
	}else if(typeof(tapFunction) == "function"){
		tapFunction(args);
	}
}

function isCollapseable(div){
	return (div && div.firstChild && div.lastChild && div.lastChild.firstChild);
}

function isGlobalCollapsed(){
	if(!globalCollapse){
		return false;
	}

	for(var i = 0; i < sections.length; i++){
		var div = document.getElementById(sections[i]);
		if(isCollapseable(div) && div.style.display != "none" && !collapsed[sections[i]]){
			return false;
		}
	}
	return true;
}

function toggleSection(args){
	var div = args[0];
	var instant = args[1];
	if(typeof(div) == "string"){
		div = document.getElementById(div);
	}
	if(!isCollapseable(div)){
		return false;
	}

	if(instant || !enableAnimations){
		div.lastChild.firstChild.style.webkitTransition = "";
	}else{
		div.lastChild.firstChild.style.webkitTransition = "all "+animation_duration+"ms ease-in-out";
	}

	if(collapsed[div.id]){
		if(globalCollapse && !instant){
			globalCollapse = 1;
		}
		div.firstChild.className = div.firstChild.className.replace("collapsed", "expanded");
		div.lastChild.firstChild.style.marginTop = "0px";
		collapsed[div.id] = false;
	}else{
		if(div.update && div.previous && div.limit != div.defaultLimit){
			div.limit = div.defaultLimit;
			div.update();
		}
		div.firstChild.className = div.firstChild.className.replace("expanded", "collapsed");
		div.lastChild.firstChild.style.marginTop = (-getSectionHeight(div))+"px";
		collapsed[div.id] = true;
	}
}

function toggleAllSections(){
	if(!globalCollapse){
		return false;
	}

	var c = !isGlobalCollapsed();

	for(var i = 0; i < sections.length; i++){
		if(c){
			lastCollapsed[sections[i]] = Boolean(collapsed[sections[i]]);
		}
		collapsed[sections[i]] = c ? false : !lastCollapsed[sections[i]];
		toggleSection([sections[i], !animateGlobalCollapse]);
	}
}

function toggleHide(){
	if(!globalHide){
		return false;
	}

	for(var i = 0; i < sections.length; i++){
		if(sections[i] == "Clock"){
			continue;
		}
		var div = document.getElementById(sections[i]);
		div.style.display = (globalHide > 0 || div.isEmpty && hideEmptySections) ? "none" : "";
	}
	globalHide -= 2*globalHide;
}

function revertCollapsed(id, subsectionPrefix){
	if(!revertCollapsedOnEmpty){
		return false;
	}

	collapsed[id] = defaultCollapsed[id];

	if(subsectionPrefix){
		for(var id in collapsed){
			if(id.indexOf(subsectionPrefix) == 0){
				delete collapsed[id];
			}
		}
	}
}

function subsectionCollapsed(id, parentID, isFirst){
	if(typeof(collapsed[id]) == "undefined"){
		collapsed[id] = expandFirst && !isFirst || !expandFirst && collapseChildren && defaultCollapsed[parentID];
	}
}

function updateTime(){
	window.clearTimeout(clockTimer);
	var now = new Date();
	document.getElementById("time").innerHTML = now.format(clock_format_time);
	clockTimer = window.setTimeout(updateTime, 1000 - (now.getTime() % 1000));
}

function updateClock(){
	var now = new Date();
	document.getElementById("date").innerHTML = now.format(format_date);

	if(showMonth){
		var today = now.getDate();

		var beginDate = new Date().zeroTime();
		beginDate.setDate(1);

		var endDate = new Date(beginDate);
		endDate.setMonth(beginDate.getMonth() + 1);

		var current = new Date(beginDate);

		current.setDate(current.getDate() + (startDay - beginDate.getDay()) - (startDay - beginDate.getDay() > 0 ? 7 : 0));

		var t = document.createElement("TABLE");
		t.id = "Month";
		t.align = "center";
		var tb = document.createElement("TBODY");
		var header = document.createElement("TR");
		for(var i = 0; i < 7; i++){
			var cell = document.createElement("TH");
			cell.width = "14%";
			cell.innerHTML = shortDays[(i + startDay) % 7];
			header.appendChild(cell);
		}
		tb.appendChild(header);
		while(current < endDate){
			var row = document.createElement("TR");
			for(var i = 0; i < 7; i++){
				var cell = document.createElement("TD");
				var date = current.getDate();
				cell.className = beginDate.getMonth() == current.getMonth() ? (today == date ? "today" : "thisMonth") : "otherMonth";
				cell.innerHTML = date;
				current.setDate(date + 1);
				row.appendChild(cell);
			}
			tb.appendChild(row);
		}
		t.appendChild(tb);
		monthDIV.innerHTML = "";
		monthDIV.appendChild(t);
	}

	updateTime();
	window.clearTimeout(dayTimer);
	dayTimer = window.setTimeout(updateClock, new Date(now.getTime() + 86400000).zeroTime().getTime() - now.getTime());

	clockDIV.potentialHeight = clockDIV.offsetHeight;

	if(collapsed["Clock"] && showMonth){
		collapsed["Clock"] = false;
		toggleSection([clockDIV, true]);
	}
}

for(var id in defaultCollapsed){
	collapsed[id] = defaultCollapsed[id];
}

window.onload = function(){
	setTimeout(updateWeather, 10);
}

document.getElementById("background").ontouchstart = catchSwipe;

for(var i = 0; i < sections.length; i++){
	switch(sections[i]){
		case "Clock":
			var now = new Date();
			clockDIV = document.createElement("DIV");
			clockDIV.id = "Clock";
			clockDIV.className = useBigClock ? "big container" : "container";
			var clockHeader = document.createElement("DIV");
			clockHeader.className = "header expanded";
			var container = document.createElement("DIV");
			container.className = "container";
			clockHeader.innerHTML = "<span id='date'></span> <span id='time'></span>";
			clockDIV.appendChild(clockHeader);
			if(showMonth){
				clockDIV.ontouchstart = function(event){catchSwipe(event, toggleSection, [this]);};
				clockHeader.innerHTML = "<div class='arrow'></div>" + clockHeader.innerHTML;
				monthDIV = document.createElement("DIV");
				monthDIV.className = "sub container";
				container.appendChild(monthDIV);
			}else{
				clockDIV.ontouchstart = catchSwipe;
			}
			clockDIV.appendChild(container);
			lockinfoDIV.appendChild(clockDIV);
			updateClock();
		break;
		case "Weather":
			weatherDIV = document.createElement("DIV");
			weatherDIV.id = "Weather";
			weatherDIV.className = "container";
			weatherDIV.style.display = "none";
			if(locales.length == 1 && mainHeaderWeather){
				weatherDIV.ontouchstart = function(event){catchSwipe(event, toggleSection, [this]);};
				var headerTouch = '';
			}else{
				var headerTouch = " ontouchstart='catchSwipe(event, toggleSection, [this.parentNode]);'";
			}
			var refresh = (!weatherUpdateInterval || displayWeatherRefreshButton) ? "<span id='weather_timestamp' class='refresh' ontouchstart='event.stopPropagation();' onclick=\"this.innerHTML='...';weatherRefresherTemp();\"></span>" : "<span id='weather_timestamp'></span>";
			var html;
			if(mainHeaderWeather){
				html = "<div id='forecast_header' class='header weather expanded'"+headerTouch+"><div class='arrow'></div>"+refresh+"<img id='weather_icon' class='icon' src='Icon Sets/none.png'/><span id='weather_city' class='city'></span><span id='weather_temp' class='temp'></span><span id='weather_desc' class='desc'></span></div><div class='container'>";
			}else{
				html = "<div id='forecast_header' class='header expanded'"+headerTouch+"><div class='arrow'></div><span id='weather_title'>"+string_weather+" ("+refresh+")</span></div><div class='container'>";
			}
			for(var j = 0; j < locales.length; j++){
				if(locales.length > 1 || !mainHeaderWeather){
					html += "<div id='weather_"+j+"' ontouchstart='catchSwipe(event, toggleSection, [this]);' class='container'><div id='forecast_"+j+"_header' class='sub1 header weather expanded'><div class='arrow'></div><img id='weather_"+j+"_icon' class='icon' src='Icon Sets/none.png'/><span id='weather_"+j+"_city' class='city'></span><span id='weather_"+j+"_temp' class='temp'></span><span id='weather_"+j+"_desc' class='desc'></span></div><div class='container'>";
				}
				html += "<div id='weather_"+j+"_forecast' class='sub container forecast'></div>";
				if(locales.length > 1){
					html += "</div></div>";
				}
			}
			weatherDIV.innerHTML = html + "</div>";
			lockinfoDIV.appendChild(weatherDIV);
		break;
		case "Mail":
			mailDIV = document.createElement("DIV");
			mailDIV.limit = mailDIV.defaultLimit = mailLimit;
			mailDIV.id = "Mail";
			mailDIV.isEmpty = true;
			mailDIV.className = "empty container";
			mailDIV.style.display = hideEmptySections ? "none" : "block";
			mailDIV.update = updateMail;
			mailDIV.innerHTML = "<div class='header expanded' ontouchstart='catchSwipe(event, toggleSection, [this.parentNode]);'><div class='arrow'></div><img src='images/mailh.png'/> "+string_mail+" (<span id='mailCount'>0</span>)</div><div class='container'><div class='container'></div></div>";
			lockinfoDIV.appendChild(mailDIV);
		break;
		case "Calendar":
			calendarDIV = document.createElement("DIV");
			calendarDIV.limit = calendarDIV.defaultLimit = calendarLimit;
			calendarDIV.id = "Calendar";
			calendarDIV.isEmpty = true;
			calendarDIV.className = "empty container";
			calendarDIV.style.display = hideEmptySections ? "none" : "block";
			calendarDIV.update = updateCalendar;
			calendarDIV.innerHTML = "<div class='header expanded' ontouchstart='catchSwipe(event, toggleSection, [this.parentNode]);'><div class='arrow'></div><img src='images/calendarh.png'/> "+string_calendar+" (<span id='calendarCount'>0</span>)</div><div class='container'><div class='container'></div></div>";
			lockinfoDIV.appendChild(calendarDIV);
		break;
		case "Calls":
			callsDIV = document.createElement("DIV");
			callsDIV.limit = callsDIV.defaultLimit = callLimit;
			callsDIV.id = "Calls";
			callsDIV.isEmpty = true;
			callsDIV.className = "empty container";
			callsDIV.ontouchstart = function(event){catchSwipe(event, toggleSection, [this]);};
			callsDIV.style.display = hideEmptySections ? "none" : "block";
			callsDIV.update = updateCalls;
			callsDIV.innerHTML = "<div class='header expanded'><div class='arrow'></div><img src='images/callsh.png'/> "+string_calls+" (<span id='callCount'>0</span>)</div><div class='container'><div class='container'></div></div>";
			lockinfoDIV.appendChild(callsDIV);
		break;
		case "Voicemail":
			voicemailDIV = document.createElement("DIV");
			voicemailDIV.limit = voicemailDIV.defaultLimit = voicemailLimit;
			voicemailDIV.id = "Voicemail";
			voicemailDIV.isEmpty = true;
			voicemailDIV.className = "empty container";
			voicemailDIV.ontouchstart = function(event){catchSwipe(event, toggleSection, [this]);};
			voicemailDIV.style.display = hideEmptySections ? "none" : "block";
			voicemailDIV.update = updateVoicemail;
			voicemailDIV.innerHTML = "<div class='header expanded'><div class='arrow'></div><img src='images/voicemailh.png'/> "+string_voicemail+" (<span id='voicemailCount'>0</span>)</div><div class='container'><div class='container'></div></div>";
			lockinfoDIV.appendChild(voicemailDIV);
		break;
		case "SMS":
			smsDIV = document.createElement("DIV");
			smsDIV.limit = smsDIV.defaultLimit = smsLimit;
			smsDIV.id = "SMS";
			smsDIV.isEmpty = true;
			smsDIV.className = "empty container";
			smsDIV.style.display = hideEmptySections ? "none" : "block";
			smsDIV.update = updateSMS;
			smsDIV.innerHTML = "<div class='header expanded' ontouchstart='catchSwipe(event, toggleSection, [this.parentNode]);'><div class='arrow'></div><img src='images/smsh.png'/> "+string_sms+" (<span id='smsCount'>0</span>)</div><div class='container'><div class='container'></div></div>";
			lockinfoDIV.appendChild(smsDIV);
		break;
	}
}

function hasClass(element, className){
	return element.className.match(new RegExp('(\\s|^)'+className+'(\\s|$)'));
}

function addClass(element, className){
	if(!this.hasClass(element, className)){
		element.className += " " + className;
	}
}

function removeClass(element, className){
	if(hasClass(element, className)){
    	var reg = new RegExp('(\\s|^)'+className+'(\\s|$)');
		element.className = element.className.replace(reg, ' ');
	}
}

if(!displayArrows){
	document.styleSheets[0].insertRule(".arrow{display:none !important;}", 0);
}

